@@ -6,11 +6,12 @@ The JWT’s claims provide a user identifier, an organization and the current ro
 """
 
 import functools
+import uuid
 from typing import Iterable
 
 import flask
 import flask_jwt_extended
-from flask_jwt_extended.exceptions import NoAuthorizationError
+import werkzeug.exceptions
 
 ctx_stack = flask._request_ctx_stack  # pylint: disable=protected-access
 get_current_session = flask_jwt_extended.get_current_user
@@ -23,13 +24,22 @@ class Session:
     def from_raw_token(cls, token: dict):
         """Create a session from raw JWT token data."""
         claims = token["user_claims"]
-        return cls(token["identity"], claims["org"], claims["roles"])
+        return cls(token["identity"], claims["org"], claims["roles"], claims["session_id"])
 
-    def __init__(self, identity: str, org: str, roles: Iterable[str] = None):
+    @classmethod
+    def create_anonymous_session(cls):
+        """Create an anonymous session from the request headers."""
+        org = flask.request.headers.get("x-ist-org", None)
+        return cls(None, org, [])
+
+    def __init__(
+        self, identity: str, org: str, roles: Iterable[str] = None, custom_uuid: str = None
+    ):
         """Create a new session instance."""
         self.identity = identity
         self.org = org
         self.roles = set(roles or [])
+        self.session_id = custom_uuid or str(uuid.uuid4())
 
     def has_any_role_of(self, roles: Iterable[str]):
         """Check if the session has any of the given roles."""
@@ -43,6 +53,7 @@ class Session:
         return {
             "identity": self.identity,
             "user_claims": {
+                "session_id": self.session_id,
                 "org": self.org,
                 "roles": list(self.roles),
             },
@@ -99,7 +110,7 @@ manager.user_identity_loader(lambda session: None)
 def check_roles(session: Session, admitted_roles: Iterable[str]):
     """Check whether the session has any of the admitted roles."""
     if not session.has_any_role_of(admitted_roles):
-        raise NoAuthorizationError("The session doesn’t have any of the required roles.")
+        raise werkzeug.exceptions.Forbidden("The user has none of the admitted roles.")
 
 
 def required(admitted_roles: Iterable[str] = None, optional=False, **kwargs):
@@ -121,7 +132,7 @@ def required(admitted_roles: Iterable[str] = None, optional=False, **kwargs):
             if flask_jwt_extended.verify_jwt_in_request(optional=optional, **decorator_kwargs):
                 session = get_current_session()
             elif optional:
-                session = Session(None, None, [])
+                session = Session.create_anonymous_session()
                 ctx_stack.top.jwt_user = {"loaded_user": session}
             manager.push_context(session)
             if session and admitted_roles is not None:
