@@ -7,6 +7,41 @@ import pytest
 from .. import jwt
 
 
+class SessionTest:
+    """Unit-tests for the Session class."""
+
+    @staticmethod
+    def test_init_generates_session_id():
+        """Test that creating a new session automatically generates a unique session id."""
+        session1 = jwt.Session("user-id", "org", [])
+        session2 = jwt.Session("user-id", "org", [])
+        assert session1.session_id
+        assert session2.session_id
+        assert session1.session_id != session2.session_id
+
+    @staticmethod
+    def test_custom_uuid_init():
+        """Test passing a custom session uuid in the constructor."""
+        uuid = "24861915-4617-4dc9-ac0e-2326c7538abc"
+        session = jwt.Session("user-id", "org", [], uuid)
+        data = session.to_token_data()
+        assert data["user_claims"]["session_id"] == uuid
+
+    @staticmethod
+    def test_create_session_from_token():
+        """Test creating a session from a token."""
+        token = {
+            "identity": "7fd8ecf2-c1fa-43fa-8661-3eafaec457b0",
+            "user_claims": {
+                "org": "org1",
+                "roles": ["admin"],
+                "session_id": "24861915-4617-4dc9-ac0e-2326c7538abc",
+            },
+        }
+        session = jwt.Session.from_raw_token(token)
+        assert session.to_token_data() == token
+
+
 @pytest.fixture(name="protected_app")
 def fixture_protected_app():
     """Create an app with one JWT protected route."""
@@ -41,6 +76,7 @@ def test_getting_session_data_in_authorized_request(protected_app):
     assert response.json["identity"] == "user-id"
     assert response.json["user_claims"]["org"] == "organization"
     assert response.json["user_claims"]["roles"] == ["app"]
+    assert response.json["user_claims"]["session_id"]
 
 
 def test_getting_injected_session_data(protected_app, jwt_inject_session):
@@ -77,3 +113,14 @@ def test_anonymous_session_with_optional(protected_app):
     assert response.json["identity"] is None
     assert response.json["user_claims"]["org"] is None
     assert response.json["user_claims"]["roles"] == []
+
+
+def test_anonymous_session_with_organization(protected_app):
+    """Test that sessions for anonymous requests read the x-ist-org header."""
+    with protected_app.test_client() as client:
+        # Showcase that the header name is not case-sensitive.
+        response = client.get("/optional", headers={"X-IsT-OrG": "test-org"})
+    assert response.status_code == 200
+    assert response.json["identity"] is None
+    assert response.json["user_claims"]["org"] == "test-org"
+    assert response.json["user_claims"]["roles"] == []
